package sjson
package json

import dispatch.json._
import JsonSerialization._

trait BasicTypes extends Protocol {
  implicit def optionFormat[T](implicit fmt : Format[T]) : Format[Option[T]] = new Format[Option[T]] {
    def writes(ot: Option[T]) = ot match {
      case Some(t) => tojson(t)
      case None => JsNull
    }
    def reads(json: JsValue) = json match {
      case JsNull => None
      case x => Some(fromjson[T](x))
    }
  }

    implicit def tuple2Format[T1,T2](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2]
    ): Format[   Tuple2[T1 ,T2 ]
] = new Format[   Tuple2[T1 ,T2 ]
]{
      def reads (json: JsValue):    Tuple2[T1 ,T2 ]
 = {
        val JsArray(e1::e2:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2)
        )
      }
      def writes(tuple:    Tuple2[T1 ,T2 ]
) = tuple match {
        case (t1,t2) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2)))
        case _ => throw new RuntimeException("Tuple" + 2 + " expected")
      }
  }
  implicit def tuple3Format[T1,T2,T3](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3]
    ): Format[   Tuple3[T1 ,T2 ,T3 ]
] = new Format[   Tuple3[T1 ,T2 ,T3 ]
]{
      def reads (json: JsValue):    Tuple3[T1 ,T2 ,T3 ]
 = {
        val JsArray(e1::e2::e3:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3)
        )
      }
      def writes(tuple:    Tuple3[T1 ,T2 ,T3 ]
) = tuple match {
        case (t1,t2,t3) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3)))
        case _ => throw new RuntimeException("Tuple" + 3 + " expected")
      }
  }
  implicit def tuple4Format[T1,T2,T3,T4](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4]
    ): Format[   Tuple4[T1 ,T2 ,T3 ,T4 ]
] = new Format[   Tuple4[T1 ,T2 ,T3 ,T4 ]
]{
      def reads (json: JsValue):    Tuple4[T1 ,T2 ,T3 ,T4 ]
 = {
        val JsArray(e1::e2::e3::e4:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4)
        )
      }
      def writes(tuple:    Tuple4[T1 ,T2 ,T3 ,T4 ]
) = tuple match {
        case (t1,t2,t3,t4) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4)))
        case _ => throw new RuntimeException("Tuple" + 4 + " expected")
      }
  }
  implicit def tuple5Format[T1,T2,T3,T4,T5](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5]
    ): Format[   Tuple5[T1 ,T2 ,T3 ,T4 ,T5 ]
] = new Format[   Tuple5[T1 ,T2 ,T3 ,T4 ,T5 ]
]{
      def reads (json: JsValue):    Tuple5[T1 ,T2 ,T3 ,T4 ,T5 ]
 = {
        val JsArray(e1::e2::e3::e4::e5:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5)
        )
      }
      def writes(tuple:    Tuple5[T1 ,T2 ,T3 ,T4 ,T5 ]
) = tuple match {
        case (t1,t2,t3,t4,t5) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5)))
        case _ => throw new RuntimeException("Tuple" + 5 + " expected")
      }
  }
  implicit def tuple6Format[T1,T2,T3,T4,T5,T6](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6]
    ): Format[   Tuple6[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ]
] = new Format[   Tuple6[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ]
]{
      def reads (json: JsValue):    Tuple6[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6)
        )
      }
      def writes(tuple:    Tuple6[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6)))
        case _ => throw new RuntimeException("Tuple" + 6 + " expected")
      }
  }
  implicit def tuple7Format[T1,T2,T3,T4,T5,T6,T7](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7]
    ): Format[   Tuple7[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ]
] = new Format[   Tuple7[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ]
]{
      def reads (json: JsValue):    Tuple7[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7)
        )
      }
      def writes(tuple:    Tuple7[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7)))
        case _ => throw new RuntimeException("Tuple" + 7 + " expected")
      }
  }
  implicit def tuple8Format[T1,T2,T3,T4,T5,T6,T7,T8](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8]
    ): Format[   Tuple8[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ]
] = new Format[   Tuple8[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ]
]{
      def reads (json: JsValue):    Tuple8[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8)
        )
      }
      def writes(tuple:    Tuple8[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8)))
        case _ => throw new RuntimeException("Tuple" + 8 + " expected")
      }
  }
  implicit def tuple9Format[T1,T2,T3,T4,T5,T6,T7,T8,T9](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9]
    ): Format[   Tuple9[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ]
] = new Format[   Tuple9[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ]
]{
      def reads (json: JsValue):    Tuple9[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9)
        )
      }
      def writes(tuple:    Tuple9[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9)))
        case _ => throw new RuntimeException("Tuple" + 9 + " expected")
      }
  }
  implicit def tuple10Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10]
    ): Format[   Tuple10[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ]
] = new Format[   Tuple10[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ]
]{
      def reads (json: JsValue):    Tuple10[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10)
        )
      }
      def writes(tuple:    Tuple10[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10)))
        case _ => throw new RuntimeException("Tuple" + 10 + " expected")
      }
  }
  implicit def tuple11Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11]
    ): Format[   Tuple11[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ]
] = new Format[   Tuple11[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ]
]{
      def reads (json: JsValue):    Tuple11[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11)
        )
      }
      def writes(tuple:    Tuple11[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11)))
        case _ => throw new RuntimeException("Tuple" + 11 + " expected")
      }
  }
  implicit def tuple12Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12]
    ): Format[   Tuple12[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ]
] = new Format[   Tuple12[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ]
]{
      def reads (json: JsValue):    Tuple12[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12)
        )
      }
      def writes(tuple:    Tuple12[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12)))
        case _ => throw new RuntimeException("Tuple" + 12 + " expected")
      }
  }
  implicit def tuple13Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13]
    ): Format[   Tuple13[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ]
] = new Format[   Tuple13[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ]
]{
      def reads (json: JsValue):    Tuple13[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13)
        )
      }
      def writes(tuple:    Tuple13[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13)))
        case _ => throw new RuntimeException("Tuple" + 13 + " expected")
      }
  }
  implicit def tuple14Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14]
    ): Format[   Tuple14[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ]
] = new Format[   Tuple14[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ]
]{
      def reads (json: JsValue):    Tuple14[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14)
        )
      }
      def writes(tuple:    Tuple14[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14)))
        case _ => throw new RuntimeException("Tuple" + 14 + " expected")
      }
  }
  implicit def tuple15Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15]
    ): Format[   Tuple15[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ]
] = new Format[   Tuple15[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ]
]{
      def reads (json: JsValue):    Tuple15[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15)
        )
      }
      def writes(tuple:    Tuple15[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15)))
        case _ => throw new RuntimeException("Tuple" + 15 + " expected")
      }
  }
  implicit def tuple16Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16]
    ): Format[   Tuple16[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ]
] = new Format[   Tuple16[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ]
]{
      def reads (json: JsValue):    Tuple16[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16)
        )
      }
      def writes(tuple:    Tuple16[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16)))
        case _ => throw new RuntimeException("Tuple" + 16 + " expected")
      }
  }
  implicit def tuple17Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17]
    ): Format[   Tuple17[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ]
] = new Format[   Tuple17[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ]
]{
      def reads (json: JsValue):    Tuple17[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17)
        )
      }
      def writes(tuple:    Tuple17[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17)))
        case _ => throw new RuntimeException("Tuple" + 17 + " expected")
      }
  }
  implicit def tuple18Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17],
      fmt18: Format[T18]
    ): Format[   Tuple18[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ]
] = new Format[   Tuple18[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ]
]{
      def reads (json: JsValue):    Tuple18[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17::e18:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17),
    fromjson[T18](e18)
        )
      }
      def writes(tuple:    Tuple18[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17),tojson(t18)(fmt18)))
        case _ => throw new RuntimeException("Tuple" + 18 + " expected")
      }
  }
  implicit def tuple19Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17],
      fmt18: Format[T18],
      fmt19: Format[T19]
    ): Format[   Tuple19[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ]
] = new Format[   Tuple19[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ]
]{
      def reads (json: JsValue):    Tuple19[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17::e18::e19:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17),
    fromjson[T18](e18),
    fromjson[T19](e19)
        )
      }
      def writes(tuple:    Tuple19[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17),tojson(t18)(fmt18),tojson(t19)(fmt19)))
        case _ => throw new RuntimeException("Tuple" + 19 + " expected")
      }
  }
  implicit def tuple20Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17],
      fmt18: Format[T18],
      fmt19: Format[T19],
      fmt20: Format[T20]
    ): Format[   Tuple20[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ]
] = new Format[   Tuple20[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ]
]{
      def reads (json: JsValue):    Tuple20[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17::e18::e19::e20:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17),
    fromjson[T18](e18),
    fromjson[T19](e19),
    fromjson[T20](e20)
        )
      }
      def writes(tuple:    Tuple20[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19,t20) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17),tojson(t18)(fmt18),tojson(t19)(fmt19),tojson(t20)(fmt20)))
        case _ => throw new RuntimeException("Tuple" + 20 + " expected")
      }
  }
  implicit def tuple21Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17],
      fmt18: Format[T18],
      fmt19: Format[T19],
      fmt20: Format[T20],
      fmt21: Format[T21]
    ): Format[   Tuple21[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ]
] = new Format[   Tuple21[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ]
]{
      def reads (json: JsValue):    Tuple21[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17::e18::e19::e20::e21:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17),
    fromjson[T18](e18),
    fromjson[T19](e19),
    fromjson[T20](e20),
    fromjson[T21](e21)
        )
      }
      def writes(tuple:    Tuple21[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19,t20,t21) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17),tojson(t18)(fmt18),tojson(t19)(fmt19),tojson(t20)(fmt20),tojson(t21)(fmt21)))
        case _ => throw new RuntimeException("Tuple" + 21 + " expected")
      }
  }
  implicit def tuple22Format[T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22](implicit 
      fmt1: Format[T1],
      fmt2: Format[T2],
      fmt3: Format[T3],
      fmt4: Format[T4],
      fmt5: Format[T5],
      fmt6: Format[T6],
      fmt7: Format[T7],
      fmt8: Format[T8],
      fmt9: Format[T9],
      fmt10: Format[T10],
      fmt11: Format[T11],
      fmt12: Format[T12],
      fmt13: Format[T13],
      fmt14: Format[T14],
      fmt15: Format[T15],
      fmt16: Format[T16],
      fmt17: Format[T17],
      fmt18: Format[T18],
      fmt19: Format[T19],
      fmt20: Format[T20],
      fmt21: Format[T21],
      fmt22: Format[T22]
    ): Format[   Tuple22[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ,T22 ]
] = new Format[   Tuple22[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ,T22 ]
]{
      def reads (json: JsValue):    Tuple22[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ,T22 ]
 = {
        val JsArray(e1::e2::e3::e4::e5::e6::e7::e8::e9::e10::e11::e12::e13::e14::e15::e16::e17::e18::e19::e20::e21::e22:: Nil) = json
        (
    fromjson[T1](e1),
    fromjson[T2](e2),
    fromjson[T3](e3),
    fromjson[T4](e4),
    fromjson[T5](e5),
    fromjson[T6](e6),
    fromjson[T7](e7),
    fromjson[T8](e8),
    fromjson[T9](e9),
    fromjson[T10](e10),
    fromjson[T11](e11),
    fromjson[T12](e12),
    fromjson[T13](e13),
    fromjson[T14](e14),
    fromjson[T15](e15),
    fromjson[T16](e16),
    fromjson[T17](e17),
    fromjson[T18](e18),
    fromjson[T19](e19),
    fromjson[T20](e20),
    fromjson[T21](e21),
    fromjson[T22](e22)
        )
      }
      def writes(tuple:    Tuple22[T1 ,T2 ,T3 ,T4 ,T5 ,T6 ,T7 ,T8 ,T9 ,T10 ,T11 ,T12 ,T13 ,T14 ,T15 ,T16 ,T17 ,T18 ,T19 ,T20 ,T21 ,T22 ]
) = tuple match {
        case (t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16,t17,t18,t19,t20,t21,t22) => JsArray(List(
      tojson(t1)(fmt1),tojson(t2)(fmt2),tojson(t3)(fmt3),tojson(t4)(fmt4),tojson(t5)(fmt5),tojson(t6)(fmt6),tojson(t7)(fmt7),tojson(t8)(fmt8),tojson(t9)(fmt9),tojson(t10)(fmt10),tojson(t11)(fmt11),tojson(t12)(fmt12),tojson(t13)(fmt13),tojson(t14)(fmt14),tojson(t15)(fmt15),tojson(t16)(fmt16),tojson(t17)(fmt17),tojson(t18)(fmt18),tojson(t19)(fmt19),tojson(t20)(fmt20),tojson(t21)(fmt21),tojson(t22)(fmt22)))
        case _ => throw new RuntimeException("Tuple" + 22 + " expected")
      }
  }
}

trait CollectionTypes extends BasicTypes with Generic {

  implicit def listFormat[T](implicit fmt : Format[T]) : Format[List[T]] = new Format[List[T]] {
    def writes(ts: List[T]) = JsArray(ts.map(t => tojson(t)(fmt)))
    def reads(json: JsValue) = json match {
      case JsArray(ts) => ts.map(t => fromjson(t)(fmt))
      case _ => throw new RuntimeException("List expected")
    }
  }

  implicit def seqFormat[T](implicit fmt : Format[T]) : Format[Seq[T]] = new Format[Seq[T]] {
    def writes(ts: Seq[T]) = JsArray(ts.toList.map(t => tojson(t)(fmt)))
    def reads(json: JsValue) = json match {
      case JsArray(ts) => ts.map(t => fromjson(t)(fmt))
      case _ => throw new RuntimeException("Seq expected")
    }
  }

  import scala.reflect.Manifest
  implicit def arrayFormat[T](implicit fmt : Format[T], mf: Manifest[T]) : Format[Array[T]] = new Format[Array[T]] {
    def writes(ts: Array[T]) = JsArray((ts.map(t => tojson(t)(fmt))).toList)
    def reads(json: JsValue) = json match {
      case JsArray(ts) => listToArray(ts.map(t => fromjson(t)(fmt)))
      case _ => throw new RuntimeException("Array expected")
    }
  }
  def listToArray[T: Manifest](ls: List[T]): Array[T] = ls.toArray

  implicit def mapFormat[K, V](implicit fmtk: Format[K], fmtv: Format[V]) : Format[Map[K, V]] = new Format[Map[K, V]] {
    def writes(ts: Map[K, V]) = JsObject(ts.map{case (k, v) => ((tojson(k.toString)).asInstanceOf[JsString], tojson(v)(fmtv))})
    def reads(json: JsValue) = json match {
      case JsObject(m) => Map() ++ m.map{case (k, v) => (fromjson[K](k)(fmtk), fromjson[V](v)(fmtv))}
      case _ => throw new RuntimeException("Map expected")
    }
  }

  import scala.collection._
  implicit def mutableSetFormat[T](implicit fmt: Format[T]): Format[mutable.Set[T]] = 
    viaSeq((x: Seq[T]) => mutable.Set(x: _*))

  implicit def immutableSetFormat[T](implicit fmt: Format[T]): Format[immutable.Set[T]] = 
    viaSeq((x: Seq[T]) => immutable.Set(x: _*))

  implicit def immutableSortedSetFormat[S](implicit ord : S => Ordered[S], binS : Format[S]) : Format[immutable.SortedSet[S]] = {
    import BasicTypes.orderable // 2.7/8 compatibility
    viaSeq((x: Seq[S]) => immutable.TreeSet[S](x: _*))
  }
}

trait StandardTypes extends CollectionTypes {
  implicit object BigIntFormat extends Format[BigInt] {
    def writes(o: BigInt) = JsValue.apply(o)
    def reads(json: JsValue) = json match {
      case JsNumber(n) => n.toBigInt
      case _ => throw new RuntimeException("BigInt expected")
    }
  }

  implicit object BigDecimalFormat extends Format[BigDecimal] {
    def writes(o: BigDecimal) = JsValue.apply(o)
    def reads(json: JsValue) = json match {
      case JsNumber(n) => n
      case _ => throw new RuntimeException("BigDecimal expected")
    }
  }
}

object BasicTypes {
  /** 2.7/8 compatibility */
  implicit def orderable[A](implicit s: A => Ordered[A]): Ordering[A] = new Ordering[A] {
    def compare(x: A, y: A) = s(x).compare(y)
  }
}
